# The MIT License (MIT)
#
# Copyright (c) 2016 Adam Schubert
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import re

try:
    from cron_descriptor import Options, ExpressionDescriptor
except ImportError:
    print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
    print('\033[1mFailed to import cron_descriptor, maybe ? "pip install cron-descriptor ?"\033[0m')
    print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
    raise


class CrontabReader(object):

    """
    Simple example reading /etc/contab
    """
    rex = re.compile(r"^(\S{1,3}\s+\S{1,3}\s+\S{1,3}\s+\S{1,3}\s+\S{1,3}).+$")

    def __init__(self, cronfile):
        """Initialize CrontabReader

        Args:
            cronfile: Path to cronfile
        Returns:
            None
        """
        options = Options()
        options.day_of_week_start_index_zero = False
        options.use_24hour_time_format = True
        with open(cronfile) as f:
            for line in f.readlines():
                parsed_line = self.parse_cron_line(line)
                if parsed_line:
                    print("{} -> {}".format(parsed_line, ExpressionDescriptor(parsed_line, options)))

    def parse_cron_line(self, line):
        """Parses crontab line and returns only starting time string

        Args:
            line: crontab line
        Returns:
            Time part of cron line
        """
        stripped = line.strip()

        if stripped and stripped.startswith('#') is False:
            rexres = self.rex.search(stripped)
            if rexres:
                return ' '.join(rexres.group(1).split())

        return None


CrontabReader('/etc/crontab')



# The MIT License (MIT)
#
# Copyright (c) 2016 Adam Schubert
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import tests.TestCase as TestCase


class TestImport(TestCase.TestCase):

    """
    Tests that inline and separated imports are working
    """

    def test_inline_import(self):
        from cron_descriptor import Options, DescriptionTypeEnum, ExpressionDescriptor
        options = Options()
        options.use_24hour_time_format = True
        options.locale_code = self.options.locale_code
        ceh = ExpressionDescriptor("* * * * *", options)
        self.assertEqual(
            "Every minute",
            ceh.get_description(DescriptionTypeEnum.FULL))

    def test_full_import(self):
        from cron_descriptor.Options import Options
        from cron_descriptor.DescriptionTypeEnum import DescriptionTypeEnum
        from cron_descriptor.ExpressionDescriptor import ExpressionDescriptor

        options = Options()
        options.use_24hour_time_format = True
        options.locale_code = self.options.locale_code
        ceh = ExpressionDescriptor("* * * * *", options)
        self.assertEqual(
            'Every minute',
            ceh.get_description(DescriptionTypeEnum.FULL))



# The MIT License (MIT)
#
# Copyright (c) 2016 Adam Schubert
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import tests.TestCase as TestCase
from cron_descriptor import Options, ExpressionDescriptor
from unittest.mock import patch
import tempfile
import shutil
import os
import logging


class TestLocale(TestCase.TestCase):

    def test_locale_de(self):
        options = Options()
        options.locale_code = 'de_DE'
        options.use_24hour_time_format = True
        self.assertEqual(
            "Jede Minute",
            ExpressionDescriptor("* * * * *", options).get_description())

    def test_locale_de_custom_location(self):
        logger = logging.getLogger('cron_descriptor.GetText')
        with patch.object(logger, "debug") as mock_logger:
            # Copy existing .mo file to temp directory:
            temp_dir = tempfile.gettempdir()
            temp_path = os.path.join(temp_dir, 'de_DE.mo')
            shutil.copyfile(os.path.join(os.path.dirname(os.path.abspath(__file__)), '../cron_descriptor/locale/', 'de_DE.mo'), temp_path)

            options = Options()
            options.locale_location = temp_dir
            options.locale_code = 'de_DE'
            options.use_24hour_time_format = True

            self.assertEqual("Jede Minute", ExpressionDescriptor("* * * * *", options).get_description())
            mock_logger.assert_called_once_with("{temp_path} Loaded".format(**{"temp_path":temp_path}))



# The MIT License (MIT)
#
# Copyright (c) 2016 Adam Schubert
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import tests.TestCase as TestCase
from cron_descriptor import DescriptionTypeEnum, ExpressionDescriptor


class TestApi(TestCase.TestCase):

    """
    Testing that API of ExpressionDescriptor is working as specified in DOCs
    """

    def test_full(self):
        self.options.use_24hour_time_format = True
        ceh = ExpressionDescriptor("* * * * *", self.options)
        self.assertEqual(
            "Every minute",
            ceh.get_description(DescriptionTypeEnum.FULL))

    def test_default(self):
        self.assertEqual(
            "Every minute",
            ExpressionDescriptor("* * * * *", self.options).get_description())

    def test_to_str(self):
        self.assertEqual(
            "Every minute",
            str(ExpressionDescriptor("* * * * *", self.options)))

    def test_to_repr(self):
        self.assertIsInstance(ExpressionDescriptor("* * * * *", self.options), ExpressionDescriptor)

    def test_to_kargs(self):
        self.assertEqual(
            "At 17:17",
            str(ExpressionDescriptor("17 17 * * *", self.options, use_24hour_time_format=True)))



# The MIT License (MIT)
#
# Copyright (c) 2016 Adam Schubert
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
import datetime

import xml.etree.ElementTree as ET
from pathlib import Path
import polib


class Resx2Po:
    def __init__(self, en_resx: Path, translation_resx: Path, code: str, output_po: Path):
        if not en_resx.is_file():
            raise Exception("EN Resx {} not found".format(en_resx.absolute()))

        if not translation_resx.is_file():
            raise Exception("Translation {} {} Resx bound not found".format(code, translation_resx.absolute()))

        self.en_resx = self.resx2dict(en_resx)
        self.translation_resx = self.resx2dict(translation_resx)
        self.code = code
        self.output_po = output_po

        self.generate()

    def resx2dict(self, resx):
        tree = ET.parse(resx)
        root = tree.getroot()
        translation_table = {}
        for first in root.findall('./data'):
            translation_table[first.attrib['name']] = first.find('./value').text

        return translation_table

    def generate(self):
        po = polib.POFile()
        now = datetime.datetime.now(datetime.timezone.utc)
        po.metadata = {
            'Project-Id-Version': '1.0',
            'Report-Msgid-Bugs-To': 'adam.schubert@sg1-game.net',
            'POT-Creation-Date': now.strftime("%Y-%m-%d %H:%M%z"),
            'PO-Revision-Date': now.strftime("%Y-%m-%d %H:%M%z"),
            'Last-Translator': 'Adam Schubert <adam.schubert@sg1-game.net>',
            'Language-Team': '',
            'MIME-Version': '1.0',
            'Content-Type': 'text/plain; charset=utf-8',
            'Content-Transfer-Encoding': '8bit',
            'Language': self.code
        }

        for message_en_id, message_en in self.en_resx.items():
            if message_en_id in self.translation_resx:
                entry = polib.POEntry(
                    msgid=message_en,
                    msgstr=self.translation_resx[message_en_id],
                    comment=message_en_id
                )
                po.append(entry)
            else:
                print('WARNING: {} not found in {} resx'.format(message_en_id, self.code))

        po.save(str(self.output_po.absolute()))


code_list = {
    'da': 'da_DK',

    # 'de': 'de_DE',
    # 'es': 'es_ES',
    # 'es-MX': 'es_MX',
    # 'fa': 'fa_IR',
    # 'fi': 'fi_FI',
    # 'fr': 'fr_FR',
    # 'it': 'it_IT',
    # 'ja': 'ja_IP',
    # 'ko': 'ko_KR',
    # 'nb': 'nb_NO',
    # 'nl': 'nl_NL',
    # 'pl': 'pl_PL',
    # 'pt': 'pt_PT',
    # 'ro': 'ro_RO',
    # 'ru': 'ru_RU',
    # 'sl': 'sl_SI',
    # 'sv': 'sv_SE',
    # 'tr': 'tr_TR',
    # 'uk': 'uk_UA',
    # 'zh-Hans': 'zh_Hans_CN',
    # 'zh-Hant': 'zh_Hant',
}

output_dir = Path('../locale')

for from_code, to_code in code_list.items():
    output_file = output_dir.joinpath('{}.po'.format(to_code))
    Resx2Po(
        Path('Resources.resx'),
        Path('Resources.{}.resx'.format(from_code)),
        to_code,
        output_file
    )
